
#include "gpioconfig.h"
#include "timeric.h"

#define NCOUNT 100
uint16_t count_buffer1[2][NCOUNT];
uint16_t count_buffer2[2][NCOUNT];
uint16_t count_index1, count_index2;
uint8_t nbuf1,nbuf2;
TimerIC *timeric;
uint16_t ccr1,lastCCR1;
uint16_t ccr2,lastCCR2;


void setup() {
  pinMode(13,OUTPUT);
  pinMode(12,OUTPUT);
  Serial.begin(115200);
  while (!Serial);
  lastCCR1 = lastCCR2 = 0;
  nbuf1 = nbuf2 = 0;
  count_index1 = count_index2 = 0;
  uint8_t polarity[2] = {POLARITY_BOTH,POLARITY_BOTH};
  uint32_t period = 10000;
  timeric = new TimerIC(8,2,period,0,polarity,false,false);
  timeric->start();
  
}

extern "C" void TIM8_CC_IRQHandler() {
    
    if (TIM8->SR & TIM_SR_CC1IF) {
      TIM8->SR &= ~ TIM_SR_CC1IF ; // Capture/compare 1 interrupt flag
      GPIOH->BSRR |= GPIO_BSRR_BS6; //digitalWrite(13,HIGH);
      ccr1 = TIM8->CCR1;
      if (ccr1>lastCCR1) count_buffer1[nbuf1][count_index1] =ccr1-lastCCR1;
      else count_buffer1[nbuf1][count_index1] =timeric->arr+1+(ccr1-lastCCR1);
      lastCCR1 = ccr1;

      count_index1++;
      if (count_index1==NCOUNT) {
        count_index1 = 0;
      if (nbuf1==1) nbuf1=0; else nbuf1=1;
      
      }
      GPIOH->BSRR |= GPIO_BSRR_BR6; //digitalWrite(13,LOW);
    }
    
    else if (TIM8->SR & TIM_SR_CC2IF) {
      TIM8->SR &= ~ TIM_SR_CC2IF ; // Capture/compare 2 interrupt flag
      GPIOJ->BSRR |= GPIO_BSRR_BS11; //digitalWrite(12,HIGH);
      ccr2 = TIM8->CCR2;
      if (ccr2>lastCCR2) count_buffer2[nbuf2][count_index2] =ccr2-lastCCR2;
      else count_buffer1[nbuf2][count_index2] =timeric->arr+1+(ccr2-lastCCR2);
      lastCCR2 = ccr2;

      count_index2++;
      if (count_index2==NCOUNT) {
        count_index2 = 0;
      if (nbuf2==1) nbuf2=0; else nbuf2=1;
      
      }
      GPIOJ->BSRR |= GPIO_BSRR_BR11; //digitalWrite(12,LOW);
    }
    
    
}

void loop() {
  delay(1000);
  Serial.println("------------------------------");
  double mean1 = 0.0;
  double mean2 = 0.0;
  double deltaT;
  uint8_t nb1,nb2;
  if (nbuf1==1) nb1=0; else nb1=1;
  if (nbuf2==1) nb2=0; else nb2=1;
  for (int i=0; i<NCOUNT; i++) {
    deltaT = count_buffer1[nb1][i]*timeric->clockPeriod;
    Serial.print(deltaT,5);
    mean1 += deltaT;
    Serial.print(", ");
    deltaT = count_buffer2[nb2][i]*timeric->clockPeriod;
    Serial.println(deltaT,5);
    mean2 += deltaT;
  }
  mean1 /= NCOUNT;
  mean2 /= NCOUNT;
  Serial.println("------------------------------");
  Serial.println(timeric->clockPeriod,5);
  Serial.print(mean1,5);
  Serial.print(", ");
  Serial.println(mean2,5);

}

    
                